package main

import (
	"fmt"
	"io/ioutil"
	"time"

	"github.com/tobiasnickel/trsa/golang/trsa"
)

func main() {
	privateKey, publicKey, err := generateKeys()
	if err != nil {
		fmt.Println(err)
		return
	}
	// fmt.Println("privateKey", len(privateKey), string(privateKey))
	ioutil.WriteFile("goPrivateKey", privateKey, 0)
	ioutil.WriteFile("goPublicKey", publicKey, 0)

	golangMessage := ""
	for i := 0; i < 100000; i++ {
		golangMessage += "Hallo Javascript from Golang"
	}
	start := makeTimestamp()
	encryptedMessage, err := trsa.Encrypt([]byte(golangMessage), publicKey)
	if err != nil {
		fmt.Println(err)
		return
	}
	fmt.Println(makeTimestamp() - start)
	ioutil.WriteFile("goEncryptedMessage", encryptedMessage, 0)
	fmt.Println("done")
}
func encryptRSA(message []byte, publicKey []byte) ([]byte, error) {
	key, err := trsa.NewKeypair(publicKey, nil)
	if err != nil {
		return nil, err
	}
	encryptedMessage, err := key.Encrypt(message)
	if err != nil {
		fmt.Println("cant encrypt")
		return nil, err
	}
	return encryptedMessage, nil
}

func generateKeys() ([]byte, []byte, error) {
	publicKey, privateKey, err := trsa.GenerateKeys(2048)
	if err != nil {
		return nil, nil, err
	}

	return privateKey, publicKey, nil
}

func makeTimestamp() int64 {
	return time.Now().UnixNano() / int64(time.Millisecond)
}
