package main

import (
	"crypto/rand"
	"crypto/rsa"
	"crypto/sha1"
	"crypto/x509"
	"encoding/pem"
	"errors"
	"fmt"
	"io/ioutil"
	"time"
)

func main() {

	encryptedMessage, err := ioutil.ReadFile("encryptedMessage") // just pass the file name
	if err != nil {
		fmt.Print(err)
	}

	privateKeyPemBytes, err := ioutil.ReadFile("privatekey") // just pass the file name
	if err != nil {
		fmt.Print(err)
	}

	plaintext, err := decryptRSA(encryptedMessage, privateKeyPemBytes)
	if err != nil {
		fmt.Print(err)
	}

	fmt.Println(string(plaintext))
}

func decryptRSA(ct []byte, privateKeyPemBytes []byte) ([]byte, error) {
	startTime := makeTimestamp()
	var privKey *rsa.PrivateKey
	var err error
	for i := 0; i < 1000; i++ {
		block, _ := pem.Decode(privateKeyPemBytes)
		if block == nil || block.Type != "RSA PRIVATE KEY" {
			return nil, errors.New("not a private key")
		}

		privKey, err = x509.ParsePKCS1PrivateKey(block.Bytes)
		if err != nil {
			return nil, err
		}
	}
	fmt.Println(makeTimestamp() - startTime)
	var plaintext []byte
	startTime = makeTimestamp()
	for i := 0; i < 1000; i++ {
		plaintext, err = rsa.DecryptOAEP(sha1.New(), rand.Reader, privKey, ct, nil)
		if err != nil {
			return nil, err
		}

	}
	fmt.Println(makeTimestamp() - startTime)

	return plaintext, nil
}

func makeTimestamp() int64 {
	return time.Now().UnixNano() / int64(time.Millisecond)
}
