package main

import (
	"bufio"
	"bytes"
	"crypto/rand"
	"crypto/rsa"
	"crypto/sha1"
	"crypto/x509"
	"encoding/pem"
	"errors"
	"fmt"
	"io/ioutil"
	"time"
)

func main() {
	var privateKey, publicKey, encryptedMessage []byte
	var err error
	start := makeTimestamp()
	for i := 0; i < 1; i++ {
		privateKey, publicKey, err = generateKeys()
		if err != nil {
			fmt.Println(err)
			return
		}
	}
	fmt.Println(makeTimestamp() - start)

	// fmt.Println("privateKey", len(privateKey), string(privateKey))
	ioutil.WriteFile("goPrivateKey", privateKey, 0)
	ioutil.WriteFile("goPublicKey", publicKey, 0)
	golangMessage := ""
	for i := 0; i < 7; i++ {
		golangMessage += "Hallo Javascript from Golang  "
	}
	golangMessage += "12345"
	fmt.Println(len(golangMessage))
	start = makeTimestamp()
	for i := 0; i < 10000; i++ {
		encryptedMessage, err = encryptRSA([]byte(golangMessage), publicKey)
		if err != nil {
			fmt.Println(err)
			return
		}
	}
	fmt.Println(makeTimestamp() - start)
	ioutil.WriteFile("goEncryptedMessage", encryptedMessage, 0)
}
func encryptRSA(message []byte, publicKey []byte) ([]byte, error) {
	block, _ := pem.Decode(publicKey)
	if block == nil || block.Type != "PUBLIC KEY" {
		return nil, errors.New("not a private key")
	}

	pubKeyInterface, err := x509.ParsePKIXPublicKey(block.Bytes)
	if err != nil {
		return nil, err
	}

	pubKey, isKey := pubKeyInterface.(*rsa.PublicKey)
	if isKey != true {
		return nil, err
	}

	encryptedMessage, err := rsa.EncryptOAEP(sha1.New(), rand.Reader, pubKey, message, nil)
	if err != nil {
		return nil, err
	}

	// fmt.Println(pubKey)
	return encryptedMessage, nil
}

func generateKeys() ([]byte, []byte, error) {
	privKey, err := rsa.GenerateKey(rand.Reader, 2048)
	if err != nil {
		return nil, nil, err
	}

	privBlock := pem.Block{}
	privBlock.Type = "RSA PRIVATE KEY"
	privBlock.Bytes = x509.MarshalPKCS1PrivateKey(privKey)

	var privateKeyBuffer bytes.Buffer
	privateKeyBufferWriter := bufio.NewWriter(&privateKeyBuffer)
	err = pem.Encode(privateKeyBufferWriter, &privBlock)
	if err != nil {
		return nil, nil, err
	}
	privateKey := make([]byte, privateKeyBufferWriter.Buffered())
	privateKeyBufferWriter.Flush()
	privateKeyBuffer.Read(privateKey)

	pubKey := privKey.Public()
	publicKeyBytes, err := x509.MarshalPKIXPublicKey(pubKey)
	if err != nil {
		return nil, nil, err
	}

	pubBlock := pem.Block{}
	pubBlock.Type = "PUBLIC KEY"
	pubBlock.Bytes = publicKeyBytes
	var publicKeyBuffer bytes.Buffer
	publicKeyBufferWriter := bufio.NewWriter(&publicKeyBuffer)
	err = pem.Encode(publicKeyBufferWriter, &pubBlock)
	if err != nil {
		return nil, nil, err
	}
	publicKey := make([]byte, publicKeyBufferWriter.Buffered())
	publicKeyBufferWriter.Flush()
	publicKeyBuffer.Read(publicKey)

	return privateKey, publicKey, nil
}

func makeTimestamp() int64 {
	return time.Now().UnixNano() / int64(time.Millisecond)
}
